==> mtc-core-v0.2.2-a69032.tar: src/Node.php <==
<?php
/**
 * Node Eloquent Model
 *
 * PHP Version 7
 *
 * @category Mtc\Core
 * @package  Mtc\Core
 * @author   Craig McCreath <craig.mccreath@mtcmedia.co.uk>
 */
namespace Mtc\Core;

use DB;
use Illuminate\Database\Eloquent\Model;
use Laravel\Scout\Searchable;
use League\Flysystem\Exception;
use Mtc\Core\CustomFields\CustomFieldValue;
use Mtc\Core\CustomFields\CustomGroup;
use Mtc\Core\Taxonomy;
use Nicolaslopezj\Searchable\SearchableTrait;

/**
 * The 'Node' is a cornerstone model that can connect to multiple different
 * models.
 *
 * It's mainly used to define a common set of fields, store a
 * unique slug which could later be used for access, and open access to
 * Custom Fields.
 *
 * @category Mtc\Core
 * @package  Mtc\Core
 * @author   Craig McCreath <craig.mccreath@mtcmedia.co.uk>
 */
class Node extends Model
{
    use Searchable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'title',
        'slug',
        'description',
        'status',
        'visibility',
        'group_id',
    ];

    /**
     * The array which is serialized by Laravel Scout to provide text based
     * searching.
     *
     * @return array
     */
    public function toSearchableArray()
    {
        return [
            'id' => $this->id,
            'title' => $this->title,
            // 'description' => $this->description,
            'nodeable_type' => $this->nodeable_type
        ];
    }

    /**
     * Register a custom save event to create slugs and generate URLs.
     *
     * @return void
     */
    public static function boot()
    {
        parent::boot();

        static::saving(
            function ($node) {
                if (empty($node->slug)) {
                    $node->createSlug(str_slug($node->title));
                } else {
                    $node->createSlug($node->slug);
                }

                if (!empty($node->nodeable)
                    && method_exists($node->nodeable, 'getUrl')
                ) {
                    $node->url = $node->nodeable->getUrl(false, $node);
                }
            }
        );
    }

    /**
     * Get a list of all owning nodeable models.
     *
     * @return \Illuminate\Database\Relations\MorphTo
     */
    public function nodeable()
    {
        return $this->morphTo();
    }

    /**
     * Retrive information about a custom group associated with this Node.
     *
     * @return \Illuminate\Database\Relations\BelongsTo
     */
    public function group()
    {
        return $this->belongsTo(CustomGroup::class, 'group_id');
    }

    /**
     * Retrieve the custom data for the current node.
     *
     * @return [type] [description]
     */
    public function getGroupDataAttribute()
    {
        $data = CustomFieldValue::ofGroup($this->group_id)
            ->ofNode($this->id)
            ->with('field')
            ->get()
            ->keyBy('field.title')
            ->map(function ($custom_value) {
                return $custom_value->value;
            })
            ->toArray();
        $data['node_id'] = $this->id;

        return $data;
    }

    /**
     * Save custom data for the current node.
     *
     * @param array $data Incoming data
     *
     * @return boolean
     */
    public function saveGroupData(array $data)
    {
        try {
            foreach ($data as $field_id => $value) {
                $custom_value = CustomFieldValue::firstOrNew([
                    'group_id' => $this->group_id,
                    'field_id' => $field_id,
                    'node_id' => $this->id
                ]);
                $custom_value->value = $value;
                $custom_value->save();
            }
            // Sync taxonomies directly to node_taxonomy for better filter performance
            //TODO: Fix taxonomy Sync
//        $this->taxonomies()->sync(
//            collect($data)
//                ->reject(
//                    function ($item, $key) {
//                        // Only return values that exist in a list
//                        $field = $this->group
//                            ->fields
//                            ->where('column_name', $key)
//                            ->where('list_id', '>', 0)
//                            ->first();
//                        if (empty($field) || empty($field->list_id)) {
//                            return true;
//                        }
//
//                        return false;
//                    }
//                )
//                ->flatten()
//                ->toArray()
//        );

        } catch (Exception $exception) {
            return false;
        }

        return true;
    }

    /**
     * Create a unique slug for the current node, set in $this->slug
     *
     * @param string $slug Slug to match against.
     *
     * @return void
     */
    public function createSlug($slug)
    {
        $base = $slug;
        $index = 1;

        // Check to see if a slug exists. If it does, then we start
        // adding an index until it's unique. E.g. hats, hats-1, hats-2.
        while ($this->slugExists($slug) == false) {
            $slug = $base . '-' . $index;
            $index++;
        }

        $this->slug = $slug;
    }

    /**
     * Determine if a slug already exists matching the provided string, except
     * for the current node.
     *
     * @param string $slug Slug to compare
     *
     * @return bool
     */
    public function slugExists($slug)
    {
        return self::whereSlug($slug)
            ->where('id', '!=', $this->id)
            ->count() == 0;
    }

    /**
     * Get the related taxonomies through the node_taxonomy pivot table
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsToMany
     */
    public function taxonomies()
    {
        return $this->belongsToMany(Taxonomy::class);
    }

    /**
     * Provide a public scope to ensure that the node is published and
     * available to the public.
     *
     * @param \Illuminate\Database\Eloquent\Builder $query
     *
     * @return \Illuminate\Database\Eloquent\Builder
     */
    public function scopePublic($query)
    {
        return $query->whereStatus('published')
            ->whereVisibility('public');
    }
}

==> mtc-core-v0.3.1-933ba8.tar: src/Node.php <==
<?php
/**
 * Node Eloquent Model
 *
 * PHP Version 7
 *
 * @category Mtc\Core
 * @package  Mtc\Core
 * @author   Craig McCreath <craig.mccreath@mtcmedia.co.uk>
 */
namespace Mtc\Core;

use DB;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\MorphTo;
use Laravel\Scout\Searchable;
use League\Flysystem\Exception;
use Mtc\Core\CustomFields\CustomFieldValue;
use Mtc\Core\CustomFields\CustomGroup;
use Mtc\Core\Taxonomy;
use Nicolaslopezj\Searchable\SearchableTrait;

/**
 * The 'Node' is a cornerstone model that can connect to multiple different
 * models.
 *
 * It's mainly used to define a common set of fields, store a
 * unique slug which could later be used for access, and open access to
 * Custom Fields.
 *
 * @category Mtc\Core
 * @package  Mtc\Core
 * @author   Craig McCreath <craig.mccreath@mtcmedia.co.uk>
 */
class Node extends Model
{
    use Searchable;

    const NODE_STATUS_PUBLISHED = 'published';
    const NODE_VISIBILITY_PUBLIC = 'public';

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'title',
        'slug',
        'description',
        'status',
        'visibility',
        'group_id',
        'custom_url',
        'url'
    ];

    /**
     * The array which is serialized by Laravel Scout to provide text based
     * searching.
     *
     * @return array
     */
    public function toSearchableArray()
    {
        return [
            'id' => $this->id,
            'title' => $this->title,
            // 'description' => $this->description,
            'nodeable_type' => $this->nodeable_type
        ];
    }

    /**
     * Register a custom save event to create slugs and generate URLs.
     *
     * @return void
     */
    public static function boot()
    {
        parent::boot();

        static::saving(
            function ($node) {

                if (empty($node->slug)) {
                    $node->createSlug(str_slug($node->title));
                } else {
                    $node->createSlug($node->slug);
                }

                if (!empty($node->nodeable)
                    && method_exists($node->nodeable, 'getUrl')
                    && $node->custom_url == 0
                ) {
                    $node->url = $node->nodeable->getUrl(false, $node);
                }
            }
        );
    }

    /**
     * Get a list of all owning nodeable models.
     *
     * @return MorphTo
     */
    public function nodeable()
    {
        return $this->morphTo();
    }

    /**
     * Retrive information about a custom group associated with this Node.
     *
     * @return \Illuminate\Database\Relations\BelongsTo
     */
    public function group()
    {
        return $this->belongsTo(CustomGroup::class, 'group_id');
    }

    /**
     * Retrieve the custom data for the current node.
     *
     * @return [type] [description]
     */
    public function getGroupDataAttribute()
    {
        $values = CustomFieldValue::ofGroup($this->group_id)
            ->ofNode($this->id)
            ->with('field')
            ->get();

        $data = [];
        foreach ($values as $row) {
            if ($row->field->multiple) {
                $data[$row->field->title][] = $row->value;
            } else {
                $data[$row->field->title] = $row->value;
            }
        }
        $data['node_id'] = $this->id;

        return $data;
    }

    /**
     * Save custom data for the current node.
     *
     * @param array $data Incoming data
     *
     * @return boolean
     */
    public function saveGroupData(array $data)
    {
        try {
            // Make all entries uniform
            foreach ($data as $field_id => $value) {
                // Multiple selects
                if (is_array($value)) {
                    // Delete entries that have been unselected
                    CustomFieldValue::where('group_id', $this->group_id)
                        ->where('field_id', $field_id)
                        ->where('node_id', $this->id)
                        ->whereNotIn('value', $value)
                        ->delete();

                    // Loop through all selected values to make sure they are saved
                    foreach ($value as $select_value) {
                        CustomFieldValue::firstOrNew([
                            'group_id' => $this->group_id,
                            'field_id' => $field_id,
                            'node_id' => $this->id,
                            'value' => $select_value
                        ])->save();
                    }
                    continue;
                } else {
                    $custom_value = CustomFieldValue::firstOrNew([
                        'group_id' => $this->group_id,
                        'field_id' => $field_id,
                        'node_id' => $this->id,
                        'value' => $value
                    ]);
                    $custom_value->save();
                }
            }

            // Sync taxonomies directly to node_taxonomy for better filter performance
            $this->taxonomies()->sync(
                // Convert input array to a collection
                collect($data)
                    ->reject(function ($item, $key) {
                        // Find fields that have this ID and have list_id
                        $field = $this->group
                            ->fields
                            ->where('id', $key)
                            ->where('list_id', '>', 0)
                            ->first();

                        // If field with ID of $key was not a taxonomy list, reject it
                        return empty($field) || empty($field->list_id);
                    })
                    // Flatten to 1D and convert to array
                    ->flatten()
                    ->toArray()
            );
        } catch (Exception $exception) {
            return false;
        }

        return true;
    }

    /**
     * Create a unique slug for the current node, set in $this->slug
     *
     * @param string $slug Slug to match against.
     *
     * @return void
     */
    public function createSlug($slug)
    {
        $base = $slug;
        $index = 1;

        // Check to see if a slug exists. If it does, then we start
        // adding an index until it's unique. E.g. hats, hats-1, hats-2.
        while ($this->slugExists($slug) == false) {
            $slug = $base . '-' . $index;
            $index++;
        }

        $this->slug = $slug;
    }

    /**
     * Determine if a slug already exists matching the provided string, except
     * for the current node.
     *
     * @param string $slug Slug to compare
     *
     * @return bool
     */
    public function slugExists($slug)
    {
        return self::whereSlug($slug)
                ->where('id', '!=', $this->id)
                ->count() == 0;
    }

    /**
     * Get the related taxonomies through the node_taxonomy pivot table
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsToMany
     */
    public function taxonomies()
    {
        return $this->belongsToMany(Taxonomy::class);
    }

    /**
     * Provide a public scope to ensure that the node is published and
     * available to the public.
     *
     * @param \Illuminate\Database\Eloquent\Builder $query
     *
     * @return \Illuminate\Database\Eloquent\Builder
     */
    public function scopePublic($query)
    {
        return $query->whereStatus(self::NODE_STATUS_PUBLISHED)
            ->whereVisibility('public');
    }
}
